#pragma once

#include "AgentDLL.h"
#include <list>

typedef int (*m_pFuncRequest)(const char*, char*, int);
//m_pFuncRequest = (int (*)(const char*, char*, int))GetProcAddress(hInst, "Request");

int RequestSEHFunc( m_pFuncRequest func, char *request, char *response, int size);

class AgentDLLCommand
{
public:
	AgentDLLCommand( const TCHAR *name, m_pFuncRequest func, const TCHAR *value = _T("") )
	{
		m_name = name;
		m_func = func;
		m_request = _T("");
		m_response = _T("");
		m_value = value;

		char request[1024];
		int size = MakeQueryXML( m_name, NULL, request, 1024 );
		m_request = request;
	}

	virtual ~AgentDLLCommand(){}
	virtual BOOL CommandImpl()
	{

		if( NULL == m_func )
			return FALSE;

		USES_CONVERSION;
		char request[1024];
		strcpy( request, T2A(m_request) );
		char response[2048];

		BOOL ret = TRUE;
		if( -1 == RequestSEHFunc( m_func, request, response, 2048 ) )
			ret = FALSE;
		m_response = response;
		return ret;
	}

	void MakeXML(CString &strXML, CString elementXML)
	{
		strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
		strXML += _T("<root>\r\n");
		strXML += elementXML;
		strXML += _T("</root>");
	}

	int MakeQueryXML(LPCTSTR qid, const CString valueXML, char* response, int response_length)
	{
		CString strElement;
		strElement = _T("<query id=\"");
		strElement += qid;
		strElement += _T("\">");
		strElement += _T("\r\n");
		if( valueXML.IsEmpty() == FALSE )
		{
			strElement += valueXML;
			strElement += _T("\r\n");
		}
		strElement += _T("</query>\r\n");
		CString strXML;
		MakeXML(strXML, strElement);
		USES_CONVERSION;
		strcpy_s(response, response_length, T2A(strXML.GetString()));
		return response_length;
	}

	BOOL Command()
	{
		return CommandImpl();
	}

	const TCHAR *GetName()
	{
		return m_name;
	}

	const TCHAR *GetResponse()
	{
		return m_response;
	}

	void SetRequest( const CString request )
	{
		m_request = request;
	}

	const TCHAR *GetRequest()
	{
		return m_request;
	}
protected:
	m_pFuncRequest	m_func;
	CString			m_response;
	CString			m_request;
	CString			m_name;
	CString			m_value;
};

class ListCtrlAdaptor
{
public:
	ListCtrlAdaptor( CListCtrl *listctrl )
	{
		m_listCtrl = listctrl;
	}

	BOOL DeleteAll()
	{
		return m_listCtrl->DeleteAllItems();
	}

	void Edit( int index, CString status, AgentDLLCommand *command )
	{
		m_listCtrl->SetItemText( index, 0, status );
		m_listCtrl->SetItemText( index, 1, command->GetName() );
		m_listCtrl->SetItemData( index, (DWORD_PTR) command );

	}

	void Add( CString status, AgentDLLCommand *command  )
	{
		int nCount = m_listCtrl->GetItemCount();
		nCount = m_listCtrl->InsertItem( nCount, status );
		m_listCtrl->SetItemText( nCount, 1, command->GetName() );
		m_listCtrl->SetItemData( nCount, (DWORD_PTR) command );
	}

protected:
	CListCtrl *m_listCtrl;
};

class AgentDLLCommandList
{
public:
	AgentDLLCommandList()
	{
		m_listctrlAdaptor = NULL;
	}

	void SetListCtrlAdaptor( ListCtrlAdaptor *listctrlAdaptor )
	{
		m_listctrlAdaptor = listctrlAdaptor;
	}

	void DeleteAll()
	{
		m_listctrlAdaptor->DeleteAll();

		CommandList::iterator it;
		for( it = m_agentCommandList.begin() ; it != m_agentCommandList.end(); )
		{
			AgentDLLCommand *command = (AgentDLLCommand *)(*it);
			if( command )
				delete command;

			it = m_agentCommandList.erase(it);
		}
	}

	void Command()
	{
		CommandList::iterator it;
		int i = 0;
		for( it = m_agentCommandList.begin() ; it != m_agentCommandList.end(); it++ )
		{
			BOOL ret = (*it)->Command();
			CString status = _T("TRUE");
			if( FALSE == ret )
				status = _T("FALSE");

			m_listctrlAdaptor->Edit( i, status, *it );
			i++;
		}
	}

	void Add(AgentDLLCommand *command)
	{
		m_agentCommandList.push_back(command);
		m_listctrlAdaptor->Add( _T(""), command );
	}

protected:
	typedef std::list<AgentDLLCommand *> CommandList;
	CommandList							m_agentCommandList;
	ListCtrlAdaptor*					m_listctrlAdaptor;
};
/*
class AgentDLLWrapper
{
public:
AgentDLLWrapper(const TCHAR *path);
~AgentDLLWrapper();

BOOL Close();

BOOL	GetInstalledStatus(NAVERAPP_INSTALL_STATUS &status, int *error_code = NULL, CString *error_msg = NULL );
BOOL	RunUpgrader(DWORD runType, int *error_code = NULL, CString *error_msg = NULL );
BOOL	GetUpdateStatus(int &status, int *error_code = NULL, CString *error_msg = NULL );
int		GetAgreementUrlList(AgreementUrlList &urlList, int *error_code = NULL, CString *error_msg = NULL );
BOOL	IsAgreementShow( int *error_code = NULL, CString *error_msg = NULL );
BOOL	AgreementShown(BOOL bShow, int *error_code = NULL, CString *error_msg = NULL );
int		GetUninstallerInfo( TCHAR* path, int pathLen, int &uninstallerType, int *error_code = NULL, CString *error_msg = NULL );
BOOL	GetGuid(TCHAR *guid, int guidSize, int *error_code = NULL, CString *error_msg = NULL );
BOOL	GetInstallError(int *error_code = NULL, CString *error_msg = NULL );
BOOL	GetUninstallError(int *error_code = NULL, CString *error_msg = NULL );
*/
